# Generated by Django 4.0.4 on 2022-05-13 12:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('codenerix_reviews', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='reviews',
            name='validate',
            field=models.BooleanField(blank=True, default=False, verbose_name='Validate'),
        ),
    ]



# -*- coding: utf-8 -*-
from django.utils.translation import gettext as _
from codenerix.forms import GenModelForm
from codenerix_reviews.models import Reviews


class ReviewsForm(GenModelForm):
    class Meta:
        model = Reviews
        exclude = []

    def __groups__(self):
        g = [
           (_('Details'), 12,
                ['customer', 6],
                ['product', 6],
                ['stars', 6],
                ['reviews', 6],
                ['lang', 6],
                ['validate', 6],
            )
        ]
        return g

    @staticmethod
    def __groups_details__():
        g = [
           (_('Details'), 12,
                ['customer', 6],
                ['product', 6],
                ['stars', 6],
                ['reviews', 6],
                ['lang', 6],
                ['validate', 6],
            )
        ]
        return g



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-05-22 13:24
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('codenerix_products', '0012_productunique_caducity'),
        ('codenerix_invoicing', '0020_auto_20180515_1333'),
    ]

    operations = [
        migrations.CreateModel(
            name='Reviews',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('updated', models.DateTimeField(auto_now=True, verbose_name='Updated')),
                ('stars', models.SmallIntegerField(validators=[django.core.validators.MaxValueValidator(10), django.core.validators.MinValueValidator(0)], verbose_name='Stars')),
                ('reviews', models.TextField(verbose_name='Reviews')),
                ('validate', models.BooleanField(default=False, verbose_name='Validate')),
                ('lang', models.CharField(choices=[('es', 'Spanish')], max_length=2, verbose_name='Language')),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reviews', to='codenerix_invoicing.Customer', verbose_name='Customer')),
                ('product', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reviews', to='codenerix_products.ProductFinal', verbose_name='Product')),
            ],
            options={
                'abstract': False,
                'default_permissions': ('add', 'change', 'delete', 'view', 'list'),
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.conf import settings
from django.db import models
from codenerix.models import CodenerixModel
from django.core.validators import MaxValueValidator, MinValueValidator
from django.utils.encoding import smart_str
from django.utils.translation import gettext_lazy as _

from codenerix_invoicing.models_sales import Customer
from codenerix_products.models import ProductFinal


# opiniones sobre productos
# El cliente podrá opinar SOLO sobre productos que haya comprado
class Reviews(CodenerixModel):
    # pasar a person
    customer = models.ForeignKey(Customer, related_name='reviews', verbose_name=_("Customer"), on_delete=models.CASCADE)
    product = models.ForeignKey(ProductFinal, related_name='reviews', verbose_name=_("Product"), on_delete=models.CASCADE)
    stars = models.SmallIntegerField(_("Stars"), validators=[MaxValueValidator(10), MinValueValidator(0)], blank=False, null=False)
    reviews = models.TextField(_("Reviews"), blank=False, null=False)
    validate = models.BooleanField(_("Validate"), blank=True, null=False, default=False)
    # idioma en el que se hace el comentario
    lang = models.CharField(_("Language"), max_length=2, choices=settings.LANGUAGES, blank=False, null=False)

    def __unicode__(self):
        return u"{} ({})".format(smart_str(self.product), smart_str(self.stars))

    def __fields__(self, info):
        fields = []
        fields.append(('customer', _("Customer")))
        fields.append(('product', _("Product")))
        fields.append(('created', _("Date")))
        fields.append(('stars', _("Stars")))
        fields.append(('reviews', _("Reviews")))
        fields.append(('lang', _("Lang")))
        fields.append(('validate', _("Validate")))
        return fields

    def save(self, *args, **kwards):
        if self.pk:
            old = Reviews.objects.get(pk=self.pk)
            product = ProductFinal.objects.get(pk=self.product.pk)
            if old.validate is False and self.validate:
                # new reviews
                value = product.reviews_value * product.reviews_count
                product.reviews_count += 1
                product.reviews_value = (value + self.stars) / product.reviews_count
                product.save()
            elif old.stars != self.stars:
                # edit reviews
                value = (product.reviews_value * product.reviews_count) - old.stars
                product.reviews_value = (value + self.stars) / product.reviews_count
                product.save()
        return super(Reviews, self).save(*args, **kwards)



# -*- coding: utf-8 -*-
from django.urls import re_path as url
from codenerix_reviews.views import ReviewsList, ReviewsUpdate, ReviewsUpdateModal, ReviewsDelete, ReviewsSubList, ReviewsDetails, ReviewsDetailModal, ReviewsCreate, ReviewsCreateModal


urlpatterns = [
    url(r'^reviews$', ReviewsList.as_view(), name='CDNX_reviews_reviewss_list'),
    url(r'^reviews/add$', ReviewsCreate.as_view(), name='CDNX_reviews_reviewss_add'),
    url(r'^reviews/addmodal$', ReviewsCreateModal.as_view(), name='CDNX_reviews_reviewss_addmodal'),
    url(r'^reviews/(?P<pk>\w+)$', ReviewsDetails.as_view(), name='CDNX_reviews_reviewss_details'),
    url(r'^reviews/(?P<pk>\w+)/edit$', ReviewsUpdate.as_view(), name='CDNX_reviews_reviewss_edit'),
    url(r'^reviews/(?P<pk>\w+)/editmodal$', ReviewsUpdateModal.as_view(), name='CDNX_reviews_reviewss_editmodal'),
    url(r'^reviews/(?P<pk>\w+)/delete$', ReviewsDelete.as_view(), name='CDNX_reviews_reviewss_delete'),
    url(r'^reviews/(?P<pk>\w+)/sublist$', ReviewsSubList.as_view(), name='CDNX_reviews_reviewss_sublist'),
    url(r'^reviews/(?P<pk>\w+)/sublist/add$', ReviewsCreateModal.as_view(), name='CDNX_reviews_reviewss_sublist_add'),
    url(r'^reviews/(?P<cpk>\w+)/sublist/(?P<pk>\w+)$', ReviewsDetailModal.as_view(), name='CDNX_reviews_reviewss_sublist_details'),
    url(r'^reviews/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/edit$', ReviewsUpdateModal.as_view(), name='CDNX_reviews_reviewss_sublist_details'),
    url(r'^reviews/(?P<cpk>\w+)/sublist/(?P<pk>\w+)/delete$', ReviewsDelete.as_view(), name='CDNX_reviews_reviewss_sublist_delete'),
]



# -*- coding: utf-8 -*-
from django.conf import settings
from django.db.models import Q
from django.utils.translation import gettext as _

from codenerix.views import GenList, GenCreate, GenCreateModal, GenUpdate, GenUpdateModal, GenDelete, GenDetail, GenDetailModal

from codenerix_reviews.models import Reviews
from codenerix_reviews.forms import ReviewsForm


# ###########################################
class GenReviewsUrl(object):
    ws_entry_point = '{}/reviews'.format(settings.CDNX_REVIEWS)


# Reviews
class ReviewsList(GenReviewsUrl, GenList):
    model = Reviews
    linkadd = False
    show_details = True
    extra_context = {'menu': ['Reviews', 'people'], 'bread': [_('Reviews'), _('People')]}


class ReviewsCreate(GenReviewsUrl, GenCreate):
    model = Reviews
    form_class = ReviewsForm
    show_details = True

    @property
    def lang(self):
        lang = None
        for x in settings.LANGUAGES:
            if x[0] == self.request.LANGUAGE_CODE:
                return self.request.LANGUAGE_CODE.lower()
        return settings.LANGUAGES[0][0].lower()

    def get_form_kwargs(self):
        kwargs = super(ReviewsCreate, self).get_form_kwargs()

        customer = self.request.user.person.customer

        kwargs['data']['customer'] = customer.pk
        kwargs['data']['lang'] = self.lang

        return kwargs


class ReviewsCreateModal(GenCreateModal, ReviewsCreate):
    pass


class ReviewsUpdate(GenReviewsUrl, GenUpdate):
    model = Reviews
    form_class = ReviewsForm
    show_details = True


class ReviewsUpdateModal(GenUpdateModal, ReviewsUpdate):
    pass


class ReviewsDelete(GenReviewsUrl, GenDelete):
    model = Reviews


class ReviewsSubList(GenReviewsUrl, GenList):
    model = Reviews
    show_details = False
    json = False
    template_model = "reviews/reviews_sublist.html"
    extra_context = {'menu': ['Reviews', 'people'], 'bread': [_('Reviews'), _('People')]}

    def __limitQ__(self, info):
        limit = {}
        pk = info.kwargs.get('pk', None)
        limit['link'] = Q(customer__pk=pk)
        return limit


class ReviewsDetails(GenReviewsUrl, GenDetail):
    model = Reviews
    groups = ReviewsForm.__groups_details__()


class ReviewsDetailModal(GenDetailModal, ReviewsDetails):
    pass
